@@ -17,11 +17,12 @@
 ##  <http://www.gnu.org/licenses/>.
 
 import codecs
-import locale
+from collections import defaultdict
 import logging
 import os
 
 import exiv2
+import chardet
 
 logger = logging.getLogger(__name__)
 
@@ -57,24 +58,18 @@ class MetadataHandler(object):
                 ):
             if prefix == 'Iptc4xmpExt' or prefix not in registered:
                 exiv2.XmpProperties.registerNs(ns, prefix)
-        # make list of possible character encodings
-        cls.encodings = ['utf-8', 'iso8859-1', 'ascii']
-        char_set = locale.getdefaultlocale()[1]
-        if char_set:
-            try:
-                name = codecs.lookup(char_set).name
-                if name not in cls.encodings:
-                    cls.encodings.append(name)
-            except LookupError:
-                pass
         
-    def __init__(self, path, buf=None):
+    def __init__(self, path=None, buf=None):
         self._path = path
         # read metadata
         if buf:
             self._image = exiv2.ImageFactory.open(buf)
         else:
             self._image = exiv2.ImageFactory.open(self._path)
+        if self._path:
+            self._name = os.path.basename(self._path)
+        else:
+            self._name = 'data'
         self._image.readMetadata()
         self._exifData = self._image.exifData()
         self._iptcData = self._image.iptcData()
@@ -95,36 +90,66 @@ class MetadataHandler(object):
             self.clear_exif()
             self.clear_iptc()
         # transcode any non utf-8 strings (Xmp is always utf-8)
-        encodings = self.encodings
-        for data in self._exifData, self._iptcData:
+        encodings = []
+        iptc_charset = self.get_iptc_encoding()
+        if iptc_charset not in ('utf-8', 'ascii', None):
+            iptc_charset = codecs.lookup(iptc_charset).name
+            logger.info('%s: IPTC character set %s', self._name, iptc_charset)
+            encodings.append(iptc_charset)
+        for data, set_value in ((self._exifData, self.set_exif_value),
+                                (self._iptcData, self.set_iptc_value)):
+            new_data = defaultdict(list)
             for datum in data:
                 if datum.typeId() not in (exiv2.TypeId.asciiString,
                                           exiv2.TypeId.string):
                     continue
-                value = datum.toString()
-                raw_value = value.encode('utf-8', errors='surrogateescape')
-                for encoding in self.encodings:
-                    try:
-                        new_value = raw_value.decode(encoding)
-                    except UnicodeDecodeError:
-                        continue
-                    if encoding != 'utf-8':
-                        logger.info('%s: transcoded %s from %s',
-                                    os.path.basename(self._path),
-                                    str(datum.key()), encoding)
-                    break
+                key = datum.key()
+                if '.0x' in key:
+                    # unknown key type
+                    continue
+                raw_value = self.get_raw_value(datum)
+                if self.decode_string(key, raw_value, 'utf-8') is not None:
+                    # no need to do anything
+                    continue
+                for encoding in encodings:
+                    value = self.decode_string(key, raw_value, encoding)
+                    if value:
+                        break
                 else:
-                    logger.warning('%s: failed to transcode %s',
-                                   os.path.basename(self._path),
-                                   str(datum.key()))
-                    new_value = raw_value.decode('utf-8', errors='replace')
-                datum.setValue(new_value)
-            iptc_charset = self.get_iptc_encoding()
-            if iptc_charset in ('utf-8', 'ascii'):
-                # no need to transcode anything
-                return
-            if iptc_charset:
-                encodings = [iptc_charset]
+                    encoding = chardet.detect(raw_value)['encoding']
+                    if encoding:
+                        try:
+                            encoding = codecs.lookup(encoding).name
+                        except LookupError:
+                            encoding = None
+                    if encoding:
+                        logger.info("%s: detected character set '%s'",
+                                    self._name, encoding)
+                        value = self.decode_string(key, raw_value, encoding)
+                    else:
+                        value = None
+                    if value:
+                        encodings.append(encoding)
+                    else:
+                        logger.warning('%s: failed to transcode %s',
+                                       self._name, key)
+                        value = raw_value.decode('utf-8', errors='replace')
+                new_data[key].append(value)
+            for key, value in new_data.items():
+                if len(value) == 1:
+                    set_value(key, value[0])
+                else:
+                    set_value(key, value)
+
+    def decode_string(self, tag, raw_value, encoding):
+        try:
+            result = raw_value.decode(encoding)
+        except UnicodeDecodeError:
+            return None
+        if encoding != 'utf-8':
+            logger.info(
+                "%s: transcoded %s from '%s'", self._name, tag, encoding)
+        return result
 
     def get_iptc_encoding(self):
         # IPTC-IIM uses ISO/IEC 2022 escape sequences to set the
@@ -132,25 +157,23 @@ class MetadataHandler(object):
         # which sequences, if any, are ever used in practice. The
         # sequence is escape, one or more intermediate bytes, then a
         # final byte.
-        iptc_charset_code = self.get_raw_value('Iptc.Envelope.CharacterSet')
-        if not iptc_charset_code or len(iptc_charset_code) < 3:
+        if 'Iptc.Envelope.CharacterSet' not in self._iptcData:
             return None
-        if iptc_charset_code[0] != 0x1b:
-            # first byte isn't escape
-            return None
-        intermediate = iptc_charset_code[1:-1]
-        final = iptc_charset_code[-1]
-        if len(intermediate) == 1:
-            if intermediate[0] == 0x25:
+        iptc_charset_code = self.get_raw_value(
+            self._iptcData['Iptc.Envelope.CharacterSet'])
+        if len(iptc_charset_code) >= 3 and iptc_charset_code[0] == 0x1b:
+            intermediate = iptc_charset_code[1:-1]
+            final = iptc_charset_code[-1]
+            if intermediate == b'\x25':
                 # "other coding systems"
                 if final == 0x47:
                     return 'utf-8'
-            if intermediate[0] in (0x28, 0x29, 0x2a, 0x2b):
+            if intermediate in (b'\x28', b'\x29', b'\x2a', b'\x2b'):
                 # "94-character set"
                 if final == 0x4e:
                     return 'koi8_r'
                 return 'ascii'
-            if intermediate[0] in (0x2c, 0x2d, 0x2e, 0x2f):
+            if intermediate in (b'\x2c', b'\x2d', b'\x2e', b'\x2f'):
                 # "96-character set"
                 if final == 0x40:
                     return 'iso_8859_5'
@@ -168,10 +191,9 @@ class MetadataHandler(object):
                     return 'iso_8859_6'
                 if final == 0x48:
                     return 'iso_8859_8'
-        elif len(intermediate) == 2:
             # "multi-byte character set"
-            if (intermediate[0] == 0x24
-                    and intermediate[1] in (0x28, 0x29, 0x2a, 0x2b)):
+            if intermediate in (b'\x24\x28', b'\x24\x29',
+                                b'\x24\x2a', b'\x24\x2b'):
                 if final == 0x42:
                     return 'euc_jp'
             if intermediate == b'\x25\x2f':
@@ -185,7 +207,7 @@ class MetadataHandler(object):
 
     def set_iptc_encoding(self):
         # escape % G, or \x1b \x25 \x47, selects UTF-8
-        self.set_value('Iptc.Envelope.CharacterSet', '\x1b%G')
+        self.set_iptc_value('Iptc.Envelope.CharacterSet', '\x1b%G')
 
     def clear_exif(self):
         self._exifData.clear()
@@ -226,20 +248,19 @@ class MetadataHandler(object):
 
     def get_exif_comment(self, datum):
         value = exiv2.CommentValue(datum.value()).comment()
-        raw_value = value.encode('utf-8', errors='surrogateescape').strip(b'\0')
         try:
-            return raw_value.decode('utf-8')
-        except UnicodeDecodeError:
+            value.encode('utf-8')
+        except UnicodeEncodeError:
             logger.error(
                 '%s: %s: %d bytes binary data will be deleted when metadata'
-                ' is saved',
-                os.path.basename(self._path), datum.key(), datum.size())
-        return None
+                ' is saved', self._name, datum.key(), datum.size())
+            return None
+        return value
 
     def get_exif_value(self, tag):
-        if tag not in self._exifData:
+        datum = self._exifData.findKey(exiv2.ExifKey(tag))
+        if datum == self._exifData.end():
             return None
-        datum = self._exifData[tag]
         if tag in ('Exif.Canon.ModelID', 'Exif.CanonCs.LensType',
                    'Exif.Image.XPTitle', 'Exif.Image.XPComment',
                    'Exif.Image.XPAuthor', 'Exif.Image.XPKeywords',
@@ -252,6 +273,10 @@ class MetadataHandler(object):
         if tag == 'Exif.Photo.UserComment' and type_id in (
                 exiv2.TypeId.comment, exiv2.TypeId.undefined):
             return self.get_exif_comment(datum)
+        if type_id == exiv2.TypeId.asciiString:
+            return datum.toString()
+        if type_id in (exiv2.TypeId.unsignedByte, exiv2.TypeId.undefined):
+            return self.get_raw_value(datum)
         if type_id == exiv2.TypeId.signedRational:
             value = exiv2.RationalValue(datum.value())
         elif type_id == exiv2.TypeId.unsignedRational:
@@ -265,32 +290,42 @@ class MetadataHandler(object):
         elif type_id == exiv2.TypeId.unsignedLong:
             value = exiv2.ULongValue(datum.value())
         else:
-            # probably a string value, so use the string
+            # unhandled type, use the string representation
+            logger.warning('%s: %s: reading %s as string',
+                           self._name, tag, datum.typeName())
             return datum.toString()
         if len(value) > 1:
             return list(value)
         return value[0]
 
+    def decode_iptc_value(self, datum):
+        type_id = datum.typeId()
+        if type_id == exiv2.TypeId.date:
+            return exiv2.DateValue(datum.value())[0]
+        if type_id == exiv2.TypeId.time:
+            return exiv2.TimeValue(datum.value())[0]
+        return datum.toString()
+
     def get_iptc_value(self, tag):
         result = None
         # findKey gets first matching datum and returns an iterator
         # which we use to search the rest of the data
         for datum in self._iptcData.findKey(exiv2.IptcKey(tag)):
-            if result:
-                if datum.key() == tag:
-                    result.append(datum.toString())
-            else:
-                result = datum.toString()
+            if result is None:
+                # first datum
+                result = self.decode_iptc_value(datum)
                 if not exiv2.IptcDataSets.dataSetRepeatable(
                                         datum.tag(), datum.record()):
                     break
                 result = [result]
+            elif datum.key() == tag:
+                result.append(self.decode_iptc_value(datum))
         return result
 
     def get_xmp_value(self, tag):
-        if tag not in self._xmpData:
+        datum = self._xmpData.findKey(exiv2.XmpKey(tag))
+        if datum == self._xmpData.end():
             return None
-        datum = self._xmpData[tag]
         type_id = datum.typeId()
         if type_id == exiv2.TypeId.xmpText:
             return datum.toString()
@@ -298,18 +333,17 @@ class MetadataHandler(object):
             return dict(exiv2.LangAltValue(datum.value()))
         return list(exiv2.XmpArrayValue(datum.value()))
 
-    def get_raw_value(self, tag):
-        if tag not in self._iptcData:
-            return None
-        datum = self._iptcData[tag]
-        return datum.toString().encode('utf-8', errors='surrogateescape')
+    def get_raw_value(self, datum):
+        result = bytearray(datum.size())
+        datum.copy(result, exiv2.ByteOrder.invalidByteOrder)
+        return result
 
     def get_exif_thumbnails(self):
         # try normal thumbnail
         thumb = exiv2.ExifThumb(self._exifData)
         data = thumb.copy()
         if data:
-            logger.info('%s: trying thumbnail', os.path.basename(self._path))
+            logger.info('%s: trying thumbnail', self._name)
             yield bytes(data), 'thumbnail'
         # try preview images
         preview_manager = exiv2.PreviewManager(self._image)
@@ -321,8 +355,7 @@ class MetadataHandler(object):
         while idx > 0:
             idx -= 1
             if max(props[idx].width_, props[idx].height_) <= 640:
-                logger.info('%s: trying preview %d',
-                            os.path.basename(self._path), idx)
+                logger.info('%s: trying preview %d', self._name, idx)
                 image = preview_manager.getPreviewImage(props[idx])
                 yield bytes(image.copy()), 'preview ' + str(idx)
 
@@ -335,10 +368,31 @@ class MetadataHandler(object):
 
     def set_exif_value(self, tag, value):
         if not value:
-            self.clear_tag(tag)
+            self.clear_exif_tag(tag)
+            return
+        key = exiv2.ExifKey(tag)
+        type_id = key.defaultTypeId()
+        if type_id == exiv2.TypeId.unsignedByte:
+            value = exiv2.DataValue(value)
+        elif type_id == exiv2.TypeId.asciiString:
+            value = exiv2.AsciiValue(value)
+        elif type_id == exiv2.TypeId.unsignedShort:
+            value = exiv2.UShortValue(value)
+        elif type_id == exiv2.TypeId.unsignedLong:
+            value = exiv2.ULongValue(value)
+        elif type_id == exiv2.TypeId.unsignedRational:
+            if isinstance(value, (list, tuple)):
+                value = exiv2.URationalValue(
+                    [(x.numerator, x.denominator) for x in value])
+            else:
+                value = exiv2.URationalValue(
+                    [(value.numerator, value.denominator)])
         else:
-            datum = self._exifData[tag]
-            datum.setValue(value)
+            # unhandled type, use the string representation
+            logger.warning('%s: %s: writing type %s as string',
+                           self._name, tag, type_id)
+        datum = self._exifData[tag]
+        datum.setValue(value)
 
     # maximum length of Iptc data
     _max_bytes = {
@@ -388,27 +442,31 @@ class MetadataHandler(object):
 
     def set_iptc_value(self, tag, value):
         # clear any existing values (which might be repeated)
-        self.clear_tag(tag)
+        self.clear_iptc_tag(tag)
         if not value:
             return
-        if isinstance(value, str):
-            # set a single value
-            datum = self._iptcData[tag]
-            datum.setValue(self.truncate_iptc(tag, value))
-            return
-        # set a list/tuple of values
+        # make list of values
         key = exiv2.IptcKey(tag)
-        for sub_value in value:
+        type_id = exiv2.IptcDataSets.dataSetType(key.tag(), key.record())
+        if type_id == exiv2.TypeId.date:
+            values = [exiv2.DateValue(*value)]
+        elif type_id == exiv2.TypeId.time:
+            values = [exiv2.TimeValue(*value)]
+        elif isinstance(value, (list, tuple)):
+            values = [self.truncate_iptc(tag, x) for x in value]
+        else:
+            values = [self.truncate_iptc(tag, value)]
+        # append values
+        while values:
             datum = exiv2.Iptcdatum(key)
-            datum.setValue(self.truncate_iptc(tag, sub_value))
+            datum.setValue(values.pop(0))
             if self._iptcData.add(datum) != 0:
-                logger.error('%s: duplicated tag %s',
-                             os.path.basename(self._path), tag)
+                logger.error('%s: duplicated tag %s', self._name, tag)
                 return
 
     def set_xmp_value(self, tag, value):
         if not value:
-            self.clear_tag(tag)
+            self.clear_xmp_tag(tag)
             return
         if '[' in tag:
             # create XMP array
@@ -440,60 +498,56 @@ class MetadataHandler(object):
         if type_id in (exiv2.TypeId.xmpAlt, exiv2.TypeId.xmpBag,
                        exiv2.TypeId.xmpSeq):
             datum.setValue(exiv2.XmpArrayValue(value, type_id))
-        elif type_id == exiv2.TypeId.langAlt:
+        elif type_id == exiv2.TypeId.langAlt or isinstance(value, dict):
             datum.setValue(exiv2.LangAltValue(value))
         else:
-            logger.error('%s: %s: setting type "%s" from %s',
-                         os.path.basename(self._path), tag,
-                         exiv2.TypeId(type_id).name, type(value))
+            logger.error(
+                '%s: %s: setting type "%s" from %s', self._name, tag,
+                exiv2.TypeId(type_id).name, type(value))
             datum.setValue(';'.join(value))
 
-    def clear_tag(self, tag):
-        data = self._data_set(tag)
-        while tag in data:
-            del data[tag]
+    def clear_exif_tag(self, tag):
+        datum = self._exifData.findKey(exiv2.ExifKey(tag))
+        if datum != self._exifData.end():
+            self._exifData.erase(datum)
+
+    def clear_iptc_tag(self, tag):
+        # findKey gets first matching datum and returns an iterator
+        # which we use to search the rest of the data
+        datum = self._iptcData.findKey(exiv2.IptcKey(tag))
+        while datum != self._iptcData.end():
+            if datum.key() == tag:
+                datum = self._iptcData.erase(datum)
+            else:
+                next(datum)
+
+    def clear_xmp_tag(self, tag):
+        datum = self._xmpData.findKey(exiv2.XmpKey(tag))
+        if datum != self._xmpData.end():
+            self._xmpData.erase(datum)
         # possibly delete Xmp container(s)
         for sep in ('/', '['):
             if sep not in tag:
                 return
             container = tag.split(sep)[0] + sep
-            for datum in data:
+            for datum in self._xmpData:
                 if datum.key().startswith(container):
                     # container is not empty
                     return
             container = tag.split(sep)[0]
-            if container in data:
-                del data[container]
+            datum = self._xmpData.findKey(exiv2.XmpKey(container))
+            if datum != self._xmpData.end():
+                self._xmpData.erase(datum)
 
     def has_iptc(self):
         return self._iptcData.count() > 0
 
-    def has_tag(self, tag):
-        data = self._data_set(tag)
-        return tag in data
+    def has_exif_tag(self, tag):
+        return tag in self._exifData
 
-    def _data_set(self, tag):
-        family = tag.split('.')[0]
-        if family == 'Exif':
-            return self._exifData
-        if family == 'Iptc':
-            return self._iptcData
-        return self._xmpData
-
-    def save_file(self, path=None):
-        if path:
-            image = exiv2.ImageFactory.open(path)
-            image.readMetadata()
-            image.setExifData(self._exifData)
-            image.setIptcData(self._iptcData)
-            image.setXmpData(self._xmpData)
-            if self._image.iccProfileDefined():
-                # copy ICC profile
-                image.setIccProfile(self._image.iccProfile())
-        else:
-            image = self._image
+    def save_file(self):
         try:
-            image.writeMetadata()
+            self._image.writeMetadata()
         except exiv2.Exiv2Error as ex:
             logger.error(str(ex))
             return False
@@ -503,11 +557,11 @@ class MetadataHandler(object):
         return True
 
     def clear_maker_note(self):
-        self.clear_tag('Exif.Image.Make')
+        self.clear_exif_tag('Exif.Image.Make')
         self._image.writeMetadata()
         self._image.readMetadata()
         self._exifData = self._image.exifData()
-        self.clear_tag('Exif.Photo.MakerNote')
+        self.clear_exif_tag('Exif.Photo.MakerNote')
 
     @staticmethod
     def create_sc(path, image_md):
