@@ -320,17 +320,6 @@ class FlickrSession(UploaderSession):
         return ''
 
 
-class PermissionWidget(DropDownSelector):
-    def __init__(self, *args, default='3'):
-        super(PermissionWidget, self).__init__(
-            *args, values = (
-                (translate('FlickrTab', 'Only you'), '0'),
-                (translate('FlickrTab', 'Friends & family'), '1'),
-                (translate('FlickrTab', 'People you follow'), '2'),
-                (translate('FlickrTab', 'Any Flickr member'), '3')),
-            default=default, with_multiple=False)
-
-
 class HiddenWidget(QtWidgets.QCheckBox):
     def set_value(self, value):
         self.setChecked(value == '2')
@@ -369,11 +358,16 @@ class TabWidget(PhotiniUploader):
         group.layout().addRow(translate('FlickrTab', 'Viewing privacy'),
                               self.widget['privacy'])
         # permissions
-        self.widget['perm_comment'] = PermissionWidget('perm_comment')
+        values = ((translate('FlickrTab', 'Only you'), '0'),
+                  (translate('FlickrTab', 'Friends & family'), '1'),
+                  (translate('FlickrTab', 'People you follow'), '2'),
+                  (translate('FlickrTab', 'Any Flickr member'), '3'))
+        self.widget['perm_comment'] = DropDownSelector(
+            'perm_comment', values=values, default='3', with_multiple=False)
         group.layout().addRow(translate('FlickrTab', 'Allow commenting'),
                               self.widget['perm_comment'])
-        self.widget['perm_addmeta'] = PermissionWidget(
-            'perm_addmeta', default='2')
+        self.widget['perm_addmeta'] = DropDownSelector(
+            'perm_addmeta', values=values, default='2', with_multiple=False)
         group.layout().addRow(translate('FlickrTab', 'Allow tags and notes'),
                               self.widget['perm_addmeta'])
         # licence
@@ -492,15 +486,13 @@ class TabWidget(PhotiniUploader):
         if size < max_size:
             return convert
         dialog = QtWidgets.QMessageBox(parent=self)
-        dialog.setWindowTitle(
-            translate('FlickrTab', 'Photini: too large'))
+        dialog.setWindowTitle(translate('FlickrTab', 'Photini: too large'))
         dialog.setText('<h3>{}</h3>'.format(
             translate('FlickrTab', 'File too large.')))
-        dialog.setInformativeText(
-            translate('FlickrTab',
-                      'File "{0}" has {1} bytes and exceeds Flickr\'s limit' +
-                      ' of {2} bytes.').format(
-                          os.path.basename(image.path), size, max_size))
+        dialog.setInformativeText(translate(
+            'FlickrTab', 'File "{0}" has {1} bytes and exceeds Flickr\'s limit'
+            ' of {2} bytes.').format(
+                os.path.basename(image.path), size, max_size))
         dialog.setIcon(dialog.Icon.Warning)
         dialog.setStandardButtons(dialog.StandardButton.Ignore)
         execute(dialog)
@@ -573,13 +565,11 @@ class TabWidget(PhotiniUploader):
         return super(TabWidget, self).replace_dialog(image, (
             ('metadata', translate('FlickrTab', 'Replace metadata')),
             ('privacy', translate('FlickrTab', 'Change viewing privacy')),
-            ('permissions',
-             translate('FlickrTab', 'Change who can comment or tag'
-                       ' (and viewing privacy)')),
-            ('safety_level',
-             translate('FlickrTab', 'Change safety level')),
-            ('hidden',
-             translate('FlickrTab', 'Change hide from search')),
+            ('permissions', translate(
+                'FlickrTab',
+                'Change who can comment or tag (and viewing privacy)')),
+            ('safety_level', translate('FlickrTab', 'Change safety level')),
+            ('hidden', translate('FlickrTab', 'Change hide from search')),
             ('licence', translate('FlickrTab', 'Change licence')),
             ('content_type', translate('FlickrTab', 'Change content type')),
             ('albums', translate('FlickrTab', 'Change album membership'))))
@@ -634,8 +624,8 @@ class TabWidget(PhotiniUploader):
         title = SingleLineEdit('title', spell_check=True)
         dialog.layout().addRow(translate('FlickrTab', 'Title'), title)
         description = MultiLineEdit('description', spell_check=True)
-        dialog.layout().addRow(translate(
-            'FlickrTab', 'Description'), description)
+        dialog.layout().addRow(translate('FlickrTab', 'Description'),
+                               description)
         dialog.layout().addRow(QtWidgets.QLabel(translate(
             'FlickrTab', 'Album will be created when photos are uploaded')))
         button_box = QtWidgets.QDialogButtonBox(
