@@ -47,10 +47,22 @@ class ComboBox(QtWidgets.QComboBox):
 
 
 class Label(QtWidgets.QLabel):
-    def __init__(self, *args, lines=1, **kwds):
+    def __init__(self, *args, lines=1, layout=None, **kwds):
         super(Label, self).__init__(*args, **kwds)
-        if lines != 1:
-            self.setText(wrap_text(self, self.text(), lines))
+        if lines == 1:
+            return
+        self.setText(wrap_text(self, self.text(), lines))
+        if not layout:
+            return
+        # match text alignment to form layout
+        align_h = (layout.labelAlignment()
+                   & Qt.AlignmentFlag.AlignHorizontal_Mask)
+        align_v = self.alignment() & Qt.AlignmentFlag.AlignVertical_Mask
+        self.setAlignment(align_h | align_v)
+        # Qt internally makes labels in form layouts 7/4 as tall, which
+        # is too much for multi=line labels
+        height = self.sizeHint().height() * ((lines * 4) + 3) // (lines * 4)
+        self.setFixedHeight(height)
 
 
 class PushButton(QtWidgets.QPushButton):
@@ -375,10 +387,10 @@ class SingleLineEdit(MultiLineEdit):
 class LatLongDisplay(SingleLineEdit):
     changed = QtSignal()
 
-    def __init__(self, image_list, *args, **kwds):
+    def __init__(self, *args, **kwds):
         super(LatLongDisplay, self).__init__('latlon', *args, **kwds)
-        self.image_list = image_list
-        self.label = QtWidgets.QLabel(self.tr('Lat, long'))
+        self.app = QtWidgets.QApplication.instance()
+        self.label = QtWidgets.QLabel(translate('LatLongDisplay', 'Lat, long'))
         self.label.setAlignment(Qt.AlignmentFlag.AlignRight)
         self.setFixedWidth(width_for_text(self, '8' * 23))
         self.setEnabled(False)
@@ -387,7 +399,7 @@ class LatLongDisplay(SingleLineEdit):
     @QtSlot(str, object)
     @catch_all
     def editing_finished(self, key, value):
-        selected_images = self.image_list.get_selected_images()
+        selected_images = self.app.image_list.get_selected_images()
         new_value = value.strip() or None
         if new_value:
             try:
@@ -403,7 +415,7 @@ class LatLongDisplay(SingleLineEdit):
 
     def update_display(self, selected_images=None):
         if selected_images is None:
-            selected_images = self.image_list.get_selected_images()
+            selected_images = self.app.image_list.get_selected_images()
         if not selected_images:
             self.set_value(None)
             self.setEnabled(False)
@@ -566,8 +578,9 @@ class LangAltWidget(QtWidgets.QWidget):
         self.lang.set_value(LangAltDict.DEFAULT)
         self.edit.set_value(self.value[LangAltDict.DEFAULT])
         lang, OK = QtWidgets.QInputDialog.getText(
-            self, self.tr('New language'), wrap_text(self, self.tr(
-                'What language is the current text in?'
+            self, translate('LangAltWidget', 'New language'),
+            wrap_text(self, translate(
+                'LangAltWidget', 'What language is the current text in?'
                 ' Please enter an RFC3066 language tag.'), 2), text=prompt)
         if OK and lang:
             self._set_default_lang(lang)
@@ -580,8 +593,9 @@ class LangAltWidget(QtWidgets.QWidget):
         if prompt in self.value:
             prompt = None
         lang, OK = QtWidgets.QInputDialog.getText(
-            self, self.tr('New language'), wrap_text(self, self.tr(
-                'What language would you like to add?'
+            self, translate('LangAltWidget', 'New language'),
+            wrap_text(self, translate(
+                'LangAltWidget', 'What language would you like to add?'
                 ' Please enter an RFC3066 language tag.'), 2), text=prompt)
         if not (OK and lang):
             return None, None
@@ -601,7 +615,7 @@ class LangAltWidget(QtWidgets.QWidget):
             return
         default_lang = self.value.get_default_lang()
         menu = QtWidgets.QMenu()
-        menu.addAction(self.tr('Set default language'))
+        menu.addAction(translate('LangAltWidget', 'Set default language'))
         for lang in langs:
             action = menu.addAction(lang)
             action.setCheckable(True)
@@ -622,12 +636,12 @@ class LangAltWidget(QtWidgets.QWidget):
     def labeled_lang(self, lang):
         if lang == LangAltDict.DEFAULT:
             if len(self.value) == 1:
-                return self.tr('Language'), lang
+                return translate('LangAltWidget', 'Language'), lang
             label = '-'
         else:
             label = lang
-        label = self.tr('Lang: ',
-                        'Short abbreviation of "Language: "') + label
+        label = translate('LangAltWidget', 'Lang: ',
+                          'Short abbreviation of "Language: "') + label
         return label, lang
 
     def set_value(self, value):
